﻿Type=Class
Version=3.71
ModulesStructureVersion=1
B4J=true
@EndOfDesignText@
'Class module
Sub Class_Globals
	Private fx As JFX
	Private frm As Form		
	Private Msgbox As Msgboxes	
	Private DialogResponse As DialogResponses		
	Private StockItemEd As TextField
	Private StockTV As TableView
	Private UnitCostEd As TextField
	Private StockID As Int		' used for Delete and edit item
	Private SelItem As String	' used for Delete and Edit item
	Private SelCost As String   ' used with Edit item
	Private IsEdit As Boolean	' Control if a Record is being Added or Updated to the Stock Table
End Sub

'Initializes the object. You can add parameters to this method if needed
Public Sub Initialize(Parent As Form)
	frm.Initialize("frm", 485, 535)
	frm.SetFormStyle("UNIFIED")
'	frm.Resizable = False
	frm.RootPane.LoadLayout("StockForm")	' Layout
	frm.SetOwner(Parent)
	frm.Stylesheets.Add(File.GetUri(File.DirAssets, "stock.css"))
	' arrange columns for Stock TableView
	SetUpStockTV
End Sub

Public Sub Show
	frm.Title = "Stock Items"	
	IsEdit = False
	' Load any Stock Items from Stock Table
	LoadStockTV
	frm.ShowAndWait	' Modal dialog
End Sub

' Window NOT resizable and keeps original Layout size
Private Sub frm_Resize(Width As Double, Height As Double)
	frm.SetWindowSizeLimits(frm.WindowWidth, frm.WindowHeight, frm.WindowWidth, frm.WindowHeight)	
End Sub

' Only Decimal Numbers accepted
Sub UnitCostEd_TextChanged (Old As String, New As String)
	Dim sb As StringBuilder
	sb.Initialize
	For i = 0 To New.Length - 1
	If "0123456789.".IndexOf(New.CharAt(i)) > -1 Then
		sb.Append(New.CharAt(i))
	End If
	Next
	UnitCostEd.Text = sb.ToString
	UnitCostEd.SetSelection(UnitCostEd.Text.Length, UnitCostEd.Text.Length)		
End Sub

' SelectedRow is used for the Delete Item Button
Sub StockTV_SelectedRowChanged(Index As Int, Row() As Object)
	If Index <> -1 Then
		StockID = Row(0)
		Dim lbl1 As Label = Row(1)
		SelItem = lbl1.text
		Dim lbl2 As Label = Row(2)
		SelCost = lbl2.text
	Else
		StockID = -1
		SelItem = ""
		SelCost = ""
	End If
	IsEdit = False
	StockItemEd.Text = ""
	UnitCostEd.Text = ""
End Sub

' Save Stock Item to Stock Table
' if IsEdit will Update a Record instead of Adding a record
Sub SaveBtn_Action
	' Validate entries in TextField
	If (StockItemEd.Text = "") Then
		Msgbox.DialogType = "ERROR"
		Msgbox.Show("Stock Item Edit Box needs to be entered! ", "Field Entry Error")	
		StockItemEd.RequestFocus
		Return	
	End If	
	If (UnitCostEd.Text = "") Then
		Msgbox.DialogType = "ERROR"
		Msgbox.Show("Unit Cost Edit Box needs to be entered! ", "Field Entry Error")	
		UnitCostEd.RequestFocus
		Return	
	End If	
	If IsEdit = False Then	
		' Add Record
		Try
			Dim M As Map
			M.Initialize
			M.Put("AId", Null)
			M.Put("Item", StockItemEd.Text)
			M.Put("Cost", UnitCostEd.Text)
			
			DBUtils.InsertMaps(DM.SQL1, "Stock", Array As Object(M))
			M.Clear
		Catch
			Msgbox.DialogType = "ERROR"
			Msgbox.Show("Error Creating Stock Item Entry! " & CRLF & LastException, "Database Error")		
		End Try
	Else
		' Edit Record
		Try
			Dim M As Map
			M.Initialize
'			M.Put("AId", Null)
			M.Put("Item", StockItemEd.Text)
			M.Put("Cost", UnitCostEd.Text)
			
			Dim where As Map
			where.Initialize
			where.Put("AId", StockID)
			
			DBUtils.UpdateRecord2(DM.SQL1, "Stock", M, where)
			M.Clear
			where.Clear
			IsEdit = False
		Catch
			Msgbox.DialogType = "ERROR"
			Msgbox.Show("Error Creating Stock Item Entry! " & CRLF & LastException, "Database Error")		
		End Try		
	End If
	StockItemEd.Text = ""
	UnitCostEd.Text = ""
	LoadStockTV
	StockItemEd.RequestFocus
End Sub

' Delete a Stock Item
Sub DeleteBtn_Action
	Dim Message, Title As String
	Dim labels() As String, types() As Int	
	Dim result As Int
	
	If StockID <> -1 Then
		' Display Message
		Message = $"Delete ${SelItem} from the Stock Grid ${CRLF} Do you wish to Continue?"$
		labels = Array As String("No", "Yes")
		types = Array As Int(DialogResponse.NEGATIVE, DialogResponse.POSITIVE)
		Msgbox.DefaultButton = DialogResponse.POSITIVE
		Title = "Delete Stock Item"
		result = Msgbox.Show3(Message, Title, labels, types)
		If result = DialogResponse.POSITIVE Then	
			Dim where As Map
			where.Initialize
			where.Put("AId", StockID)
			DBUtils.DeleteRecord(DM.SQL1, "Stock", where)
			LoadStockTV
		End If		
	Else
		Msgbox.DialogType = "ERROR"
		Msgbox.Show("Stock Item needs to be Selected in Stock Grid! ", "Stock Grid Selection Error")	
	End If
End Sub

' Edit a Stock item entry - EditBtn click
Sub EditBtn_Action
	If StockID <> -1 Then
		StockItemEd.Text = SelItem
		UnitCostEd.Text = SelCost
		IsEdit = True
	End If	
End Sub

' Load the StockTV from the Stock Table
Sub LoadStockTV
	DBUtils.LoadStockTable(DM.SQL1, "SELECT * FROM Stock ORDER BY Item", Null, 0, StockTV)
	If StockTV.Items.Size > 0 Then StockTV.SelectedRow = 0
End Sub

' Setup the Columns for the StockTV 
Sub SetUpStockTV
	Dim RList As List
		
	RList.Initialize
	RList.Add("AId")
	RList.Add("Stock Item")
	RList.Add("Unit Cost")
	
	StockTV.SetColumns(RList)
	
	StockTV.SetColumnVisible(0, False) 	' AId
	StockTV.SetColumnWidth(1, 360.01)	' Stock Item
	StockTV.SetColumnWidth(2, 84.01)	' Unit Cost
		
	For i= 1 To 2
		StockTV.SetColumnSortable(i, False)
	Next
	
	RList.Clear			
End Sub


