﻿Type=Class
Version=3.71
ModulesStructureVersion=1
B4J=true
@EndOfDesignText@
'Class module
Sub Class_Globals
	Private fx As JFX
	Private frm As Form			
	Private Msgbox As Msgboxes
	Private DialogResponse As DialogResponses				
	Private AcntMap As Map					' Holds List of Accounts Expected Heights when Printed
	Private AcntList As List				' Holds List of Accounts (Acnt) Table
	Dim SummaryList As List					' Holds List of Summary for an Account (AcntSmry) Table
	Private NumberOfAcnts As Int			' Holds Number of Accounts in Account Table (Acnt)
	Dim CAcnt() As String					' Data from Current Account in use (Acnt)
	Private AcntIdx As Int					' Index of AcntList - used to progress through Accounts (Acnt)
	Private CurAcntId As Int				' Holds the Current Account Id (MId) - used as Foreign Key in Summary table (AcntSmry)
	Private Y_Axis As Float					' Variable used to Locate the Vertical Position to Draw Text and Lines
	Private CurrentDepth As Float			' Hold Current Depth on page - Compares to the Total Depth (820) before Next Page is required
	Private PdfFileName As String			' File name to save the generated PDF inot
	' PDFjet Vars
	Private PDFjetConstants1 As PDFjetConstants
	Private PDFjetPDF1 As PDFjetPDF
	Dim PDFPage As PDFjetPage
	Dim PDFFont1 As PDFjetFont
	Dim PDFFont2 As PDFjetFont
	Dim PDFFont3 As PDFjetFont		
End Sub

'Initializes the object. You can add parameters to this method if needed.
Public Sub Initialize(Parent As Form)
	frm.Initialize("frm", 440, 240)
	frm.SetFormStyle("UNIFIED")
'	frm.Resizable = False
	frm.RootPane.LoadLayout("SummaryForm")	' Layout
	frm.SetOwner(Parent)
	frm.Stylesheets.Add(File.GetUri(File.DirAssets, "sumry.css"))
End Sub

Public Sub Show
	frm.Title = "Account Summary"	
	frm.ShowAndWait		' Modal dialog
End Sub

' Window NOT resizable and keeps original Layout size
Private Sub frm_Resize(Width As Double, Height As Double)
	frm.SetWindowSizeLimits(frm.WindowWidth, frm.WindowHeight, frm.WindowWidth, frm.WindowHeight)	
End Sub

' Print the Account and its Summary of Invoices
Sub PrintSumBtn_Action
	' check there are Accounts via BuildAcntList
	If BuildAcntList Then
		' Buid Map of all Accounts and the number of Rows of Summary
		BuildAcntMap
		' Create the PDF
		CreatePDFTable
	End If
End Sub

' Delete all the Account Summary Table (AcntSmry)
Sub DeleteSumBtn_Action
	Dim Message, Title As String
	Dim labels() As String, types() As Int	
	Dim result As Int

	' Display Message
	Message = $"All Account Summary Entries will be Deleted from Database!${CRLF} Do you wish to Continue?"$
	labels = Array As String("No", "Yes")
	types = Array As Int(DialogResponse.NEGATIVE, DialogResponse.POSITIVE)
	Msgbox.DefaultButton = DialogResponse.POSITIVE
	Title = "Delete Account Summary"
	result = Msgbox.Show3(Message, Title, labels, types)
	If result = DialogResponse.POSITIVE Then
		' Drop the AcntSmry Table
		DBUtils.DropTable(DM.SQL1, "AcntSmry")
		' Create the AcntSmry Table
		DM.CreateAcntSummary
	End If
End Sub

' Setup the base of the PDF Doc
Sub CreateBasePDFDocument As PDFjetPDF
	Dim PDFDestination As OutputStream
	
	PdfFileName = "AcSummary.pdf"
	
	PDFDestination = File.OpenOutput(DM.ExcelFilePath, PdfFileName, False)
	
	PDFjetPDF1.Initialize("PDFjetPDF1", PDFDestination)
	PDFjetPDF1.SetAuthor("Black Jack")
	PDFjetPDF1.SetSubject("BPak Produce Merchant")
	PDFjetPDF1.SetTitle("Account Summary")

	Return PDFjetPDF1
End Sub

' Main Sub to do the processing of the Drawing into the PDF Doc
Sub CreatePDFTable
	Dim PDFjetPDF1 As PDFjetPDF = CreateBasePDFDocument

	' set Page Size and Orientation and fonts
	SetPageParams	
	
	' Draw the tilte on the Page
	Y_Axis = 15
	SetDocumentTitle

	Y_Axis = 35				' start acnt name first page

	' Repeat through Accounts
	Do While AcntIdx < NumberOfAcnts
		' Obtain data for current Account
		ObtainCurAcnt
		
		' Print Account info to Page
		MakeAcntHead
		
		' Obtain data from Summary (AcntSmry) for the Current Account
		BuildSummaryList		
		
		' Iterate through the Account Summary for this Curren Account
		Y_Axis = Y_Axis + 30		' top of summary
		Y_Axis = ProcessSummaryData(Y_Axis)
		Y_Axis = Y_Axis + 15		' add space between Acnts
		
		' Check that the Page is able to take the next race
		' if not create a New Page
		If AcntIdx < NumberOfAcnts Then
			CurrentDepth = Y_Axis
			Dim RDepth As Float = AcntMap.Get(AcntIdx) 	' Expected Depth of Next Account with its Summary
			If CurrentDepth + RDepth > 785.0 Then
				PDFPage.Initialize(PDFjetPDF1, PDFjetConstants1.PageSize.A4_PORTRAIT)
				Y_Axis = 25
			End If
		End If		
	Loop
	
	PDFjetPDF1.Close
End Sub

' Sets the PDF Page Size, and Fonts needed
Sub SetPageParams
	PDFPage.Initialize(PDFjetPDF1, PDFjetConstants1.PageSize.A4_PORTRAIT)
	
	PDFFont1.Initialize(PDFjetPDF1, PDFjetConstants1.CoreFont.HELVETICA_BOLD)
	PDFFont1.SetSize(9)

	PDFFont2.Initialize(PDFjetPDF1, PDFjetConstants1.CoreFont.HELVETICA)
	PDFFont2.SetSize(9)
	
	PDFFont3.Initialize(PDFjetPDF1, PDFjetConstants1.CoreFont.TIMES_BOLD)
	PDFFont3.SetSize(30)
	
End Sub

' Draw Heading on Page
Sub SetDocumentTitle
	PDFFont3.SetSize(20)
	
	DrawTextItem(PDFFont3, 30, Y_Axis, 542, 25, "BPak Produce Merchant - Account Summary", PDFjetConstants1.Color.crimson, PDFjetConstants1.Align.CENTER)	
End Sub

' Draw the Account info on page
Sub MakeAcntHead
	Dim Str1 As String
	
	Str1 = $"${CAcnt(1)} - Phone: ${CAcnt(2)}"$
	
	PDFFont1.SetSize(12)	
	DrawTextItem(PDFFont1, 40, Y_Axis + 12, 400, 20, Str1, PDFjetConstants1.Color.blue, PDFjetConstants1.Align.LEFT)
End Sub

' Builds a List of Summary (AcntSmry) for the Current Account
Sub BuildSummaryList
	Dim Qry1 As String
	Dim SummaryList As List
	
	If SummaryList.IsInitialized Then SummaryList.Clear
	
	' AcntID, InvNo As Int, SDate As String, SubTotal, Tax, Total As Double)
	
	Qry1 = $"
	SELECT SDate, InvNo, SubTotal, Tax, Total  
	FROM AcntSmry
	WHERE AcntID = ? 
	"$
	SummaryList = DBUtils.ExecuteMemoryTable(DM.SQL1, Qry1, Array As String(CurAcntId), 0)	
End Sub

' Draws all the Summary entries for each Account
Sub ProcessSummaryData(V As Float) As Float
	Dim StartTop, EndBottom As Float
	Dim bal As Double
	
	PDFFont1.SetSize(10)
	PDFFont2.SetSize(10)
				
	bal = 0.0			
	If SummaryList.Size > 0 Then
		' Start Point for Summary Table Heading and used as start point for Vertical Rules
		StartTop = V
		' Horiz Line above Summary Heading
		Dim HL As PDFjetLine
		HL.Initialize(40.0, StartTop, 560.0, StartTop)
		HL.DrawOn(PDFPage)
		
		' Draw Heading for Summary entry
		DrawTextItem(PDFFont1, 40, V, 80, 15, "Date", PDFjetConstants1.Color.black, PDFjetConstants1.Align.CENTER)
		DrawTextItem(PDFFont1, 120, V, 150, 15, "Invoice Number", PDFjetConstants1.Color.black, PDFjetConstants1.Align.CENTER)
		DrawTextItem(PDFFont1, 270, V, 70, 15, "Sub-Total", PDFjetConstants1.Color.black, PDFjetConstants1.Align.CENTER)
		DrawTextItem(PDFFont1, 340, V, 70, 15, "Tax", PDFjetConstants1.Color.black, PDFjetConstants1.Align.CENTER)
		DrawTextItem(PDFFont1, 410, V, 70, 15, "Total", PDFjetConstants1.Color.black, PDFjetConstants1.Align.CENTER)
		DrawTextItem(PDFFont1, 480, V, 80, 15, "Balance", PDFjetConstants1.Color.black, PDFjetConstants1.Align.CENTER)

		' Horiz line below Summary heading
		HL.Initialize(40.0, V+14, 560.0, V+14)
		HL.DrawOn(PDFPage)
	
		' Draw all Rows of the Summary entry
		For j = 0 To SummaryList.Size - 1
			Dim Col() As String
			Col = SummaryList.Get(j)
			bal = bal + Col(4)
			V = V + 14
			DrawTextItem(PDFFont2, 40, V, 80, 15, Col(0), PDFjetConstants1.Color.black, PDFjetConstants1.Align.CENTER)
			DrawTextItem(PDFFont2, 120, V, 150, 15, Col(1), PDFjetConstants1.Color.black, PDFjetConstants1.Align.CENTER)
			DrawTextItem(PDFFont2, 270, V, 70, 15, NumberFormat2(Col(2), 1, 2, 2, False), PDFjetConstants1.Color.black, PDFjetConstants1.Align.CENTER)
			DrawTextItem(PDFFont2, 340, V, 70, 15, NumberFormat2(Col(3), 1, 2, 2, False), PDFjetConstants1.Color.black, PDFjetConstants1.Align.CENTER)
			DrawTextItem(PDFFont2, 410, V, 70, 15, NumberFormat2(Col(4), 1, 2, 2, False), PDFjetConstants1.Color.black, PDFjetConstants1.Align.CENTER)
			DrawTextItem(PDFFont2, 480, V, 80, 15, NumberFormat2(bal, 1, 2, 2, False), PDFjetConstants1.Color.black, PDFjetConstants1.Align.CENTER)
		Next	

		' Position for Bottom Rule and end point for Vertical Rules
		EndBottom = V + 14
		
		' Horiz Line at bottom of Summary entries
		HL.Initialize(40.0, EndBottom, 560.0, EndBottom)
		HL.DrawOn(PDFPage)
		
		' Vertical Lines
		HL.Initialize(120.0, StartTop, 120.0, EndBottom)
		HL.DrawOn(PDFPage)
		
		HL.Initialize(270.0, StartTop, 270.0, EndBottom)
		HL.DrawOn(PDFPage)
		
		HL.Initialize(340.0, StartTop, 340.0, EndBottom)
		HL.DrawOn(PDFPage)
		
		HL.Initialize(410.0, StartTop, 410.0, EndBottom)
		HL.DrawOn(PDFPage)
		
		HL.Initialize(480.0, StartTop, 480.0, EndBottom)
		HL.DrawOn(PDFPage)
		
	End If
	Return V
End Sub

' This Sub is used to Draw all the TEXT from all the other Subs
Sub DrawTextItem(PDFFont As PDFjetFont, LPos As Int, VPos As Int, Width As Int, Height As Int, Str1 As String, clr As Int, aln As Int)	
	Dim PDFTextBox As PDFjetTextBox
	PDFTextBox.Initialize(PDFFont)
	PDFTextBox.SetPosition(LPos, VPos)
	PDFTextBox.SetWidth(Width)
	PDFTextBox.SetHeight(Height)
	PDFTextBox.SetText(Str1)
	PDFTextBox.SetFgColor(clr)
	PDFTextBox.SetTextAlignment(aln)
	PDFTextBox.SetNoBorders					' comment out to see Draw Boxes for containing text

	PDFTextBox.DrawOn(PDFPage)	
End Sub	

' Obtains the Current Account to be printed
Sub ObtainCurAcnt
	If AcntIdx < NumberOfAcnts Then
		Dim CAcnt() As String
		CAcnt = AcntList.Get(AcntIdx)
		CurAcntId = CAcnt(0)
		AcntIdx = AcntIdx + 1
	End If	
End Sub

' Builds the Account List
Sub BuildAcntList As Boolean
	Dim Qry1 As String
	Dim Fnd As Boolean = False

	AcntList.Initialize
	
	If AcntList.IsInitialized Then AcntList.Clear
		
	Qry1 = $"
	SELECT MId, Name, Phone  
	FROM Acnt
	ORDER BY Name 
	"$
	AcntList = DBUtils.ExecuteMemoryTable(DM.SQL1, Qry1, Null, 0)
	If AcntList.Size > 0 Then
		Fnd = True	
		NumberOfAcnts = AcntList.Size
		AcntIdx = 0
	Else
		Fnd = False
	End If
	
	Return Fnd
End Sub

' Creates a Map of all Accounts including the Summary Lines and holds total depth for this entry
Sub BuildAcntMap
	Dim AcID, cnt As Int
	Dim Qry1 As String
	
	AcntMap.Initialize
	
	If AcntList.Size > 0 Then
		For i = 0 To AcntList.Size - 1
			Dim AR() As String
			AR = AcntList.Get(i)

			AcID = AR(0)
			
	    	Qry1 = $"
			SELECT Count(*) FROM 
			AcntSmry 
			WHERE AcntID = ? 
			"$
			
			cnt = DM.SQL1.ExecQuerySingleResult2(Qry1, Array As String(AcID))			
			cnt = (cnt * 14) + 20
			AcntMap.Put(i, cnt)			
		Next
	End If
End Sub

' Display the Generated PDF Page/s - Print option is then accessible from the PDF Doc
Sub PDFjetPDF1_CloseComplete(Success As Boolean)
	If Success Then
		fx.ShowExternalDocument(File.GetUri(DM.ExcelFilePath, PdfFileName))
	Else
		Log("An error has occurred and creation of the PDF document has failed")
	End If
End Sub


